MODULE defs
  REAL*8,PARAMETER             :: zero= 0.0d0
  COMPLEX*16,PARAMETER         :: zeroc= (0.0d0,0.0d0)
END MODULE defs

MODULE struct
  LOGICAL     :: ortho,inversion
  INTEGER     :: nat,nsym,ndif
  CHARACTER*4  :: lattic,irel,cform
  CHARACTER*80 :: title
  CHARACTER*10,ALLOCATABLE :: aname(:)
  REAL*8      :: br1(3,3),aa,bb,cc,br2(3,3),avec(3,3)
  REAL*8      :: a(3),alpha(3),pia(3),vol
  INTEGER,ALLOCATABLE :: iatnr(:),mult(:),jri(:),isplit(:)      !NATO
  REAL*8,ALLOCATABLE  :: r0(:),dx(:)                            !NATO   
  REAL*8,ALLOCATABLE  :: rotloc(:,:,:),rmt(:),zz(:),vsph(:)     !NATO   
  REAL*8,ALLOCATABLE  :: rotij(:,:,:),tauij(:,:)                !NDIF
  REAL*8,POINTER      :: pos(:,:)
END MODULE struct

MODULE symetr
  INTEGER   :: iord
  INTEGER,ALLOCATABLE :: iz(:,:,:),inum(:),kkk(:,:) !NSYM
  INTEGER,ALLOCATABLE :: kzz(:,:),inst(:)           !NVAW
  REAL*8,ALLOCATABLE  :: tau(:,:)                   !NSYM
END MODULE symetr

MODULE parallel
  INTEGER             :: ierr,npe,myid,npe_coul,myid_coul
  INTEGER,ALLOCATABLE :: nstart(:),nstop(:)
  INTEGER             :: comm_coul,comm_xc
  INTEGER             :: icoul
  INTEGER,ALLOCATABLE :: statusmpi(:)
  LOGICAL             :: coul,xc
  
  CONTAINS
  
  SUBROUTINE init_parallel
#ifdef Parallel
    IMPLICIT NONE
    INCLUDE 'mpif.h'
    ALLOCATE(statusmpi(MPI_STATUS_SIZE))
    CALL MPI_INIT(ierr)
    IF(ierr.NE.MPI_SUCCESS) THEN
       PRINT*, 'MPI ini error'
       call abort_parallel
       STOP
    ENDIF
    CALL MPI_COMM_SIZE( MPI_COMM_WORLD, npe, ierr)
    IF(ierr.NE.MPI_SUCCESS) THEN
       PRINT*, 'MPI_comm_size error'
       call abort_parallel
       STOP
    ENDIF
    CALL MPI_COMM_RANK( MPI_COMM_WORLD, myid, ierr)
#else
    myid=0
    npe=1
    myid_coul=0
    npe_coul=1
#endif
    ALLOCATE(nstart(0:npe-1),nstop(0:npe-1))
  END SUBROUTINE init_parallel

        SUBROUTINE abort_parallel
#ifdef Parallel
        IMPLICIT NONE
        include 'mpif.h'
        INTEGER :: IERR
!       Flush outputs
        call flush(6)
        close(unit=6)
        if(myid .eq. 0)then
                call flush(99)
                close(unit=99)
        endif
        CALL MPI_ABORT(MPI_COMM_WORLD, IERR)
#endif
        END SUBROUTINE abort_parallel

  SUBROUTINE begend(nat)
#ifdef Parallel
    IMPLICIT NONE
    INCLUDE 'mpif.h'
    INTEGER nat,modu,i,icolor1
    icolor1=MPI_UNDEFINED
    coul=.FALSE.
    xc=.true.
    modu=(nat-1)/npe+1
    icoul=0
    icolor1=MPI_UNDEFINED
    DO i=0,npe-1
       nstart(i)=i*modu+1
       nstop(i)=MIN(nat,(i+1)*modu)
       IF(nstart(i).LE.nstop(i)) icoul=i
    ENDDO
    IF(nstart(myid).LE.nstop(myid)) THEN
       icolor1=1
       coul=.true.
    ENDIF

    call MPI_COMM_SPLIT(MPI_COMM_WORLD, icolor1, myid, comm_coul,ierr)

    if (nstart(myid).LE.nstop(myid)) then
       CALL MPI_COMM_SIZE( comm_coul, npe_coul, ierr)
       CALL MPI_COMM_RANK( comm_coul, myid_coul, ierr)
    else
       npe_coul=0
       myid_coul=myid
    endif

!!$    icolor2=MPI_UNDEFINED
!!$    IF(myid.LT.nat) THEN
!!$       DO i=
!!$       nstart=myid*modu+1
!!$       nstop=MIN(nat,(myid+1)*modu)
!!$       icolor1=1
!!$       IF(nstop.EQ.nat) log_write=.TRUE.
!!$       IF(nstart.gt.nstop) THEN
!!$          icolor1=MPI_UNDEFINED
!!$          coul=.false.
!!$       ENDIF
!!$!       icolor2=1
!!$    endif
!!$    elseif(npe.ge.(2*nat)) then
!!$       icoul=nat-1
!!$       ixc=2*nat-1
!!$!       write(6,*) 'icoul,ixc',icoul,ixc
!!$       if(myid.lt.nat) then
!!$          modu=(nat-1)/(npe-nat)+1
!!$          nstart=myid*modu+1
!!$          nstop=MIN(nat,(myid+1)*modu)
!!$          icolor1=1
!!$       elseif(myid.ge.nat.and.myid.lt.(2*nat)) then
!!$          modu=(nat-1)/(npe-nat)+1
!!$          nstart=(myid-nat)*modu+1
!!$          nstop=MIN(nat,((myid-nat)+1)*modu)
!!$          icolor2=1
!!$      endif
!!$    endif
!    call MPI_COMM_SPLIT(MPI_COMM_WORLD, icolor2, myid, comm_xc,ierr)
#endif
  END SUBROUTINE begend
END MODULE parallel



