      program scf2forces
!
! This program will apply the symmetry operations to forces from case.scf 
! and write a new FORCES file for the phonon program 'phonopy'.
! 'phonopy' limit = works only with P symmetry (1 symmetry operation)
! 'wien2k'efficiency = converts supercells structures created by 'phonopy' 
! to structures with higher symmetry
!
      use struct
      use symetr
      use defs
!
      implicit none
      integer :: i, j, k, m, ndisp, where1, where2 
      integer :: nindex,lnam,length
      real*8, allocatable :: forces(:,:),forces_new(:,:) 
      character*100 :: line, line1, fname, fname1
      character*80 :: opennumber, opennumber1
!  
      open(1,file='FORCES',status='old',err=1001)
      open(2,file='FORCES_new',err=1001)
!
!      fname = 'case_S-' 
      write(*,'(a)')'Please, type name of your &  
      nameXXX.struct(scf) files:'
      write(*,'(a)')'(XXX=001,002,...,099,...,999 &
      supercell calculations)' 
      read(*,'(a)')fname
      lnam = len(trim(fname))
      read(1,*)ndisp
      write(*,'(a25,i3)')'number of displacements =',ndisp
      write(2,'(i4)')ndisp
      do i = 1,ndisp 
        if(i.lt.10) then
          write(opennumber,'("00",i1,".struct")') i
          write(opennumber1,'("00",i1,".scf")') i
        elseif(i.lt.100) then
          write(opennumber,'("0",i2,".struct")') i
          write(opennumber1,'("0",i2,".scf")') i
        else
          write(opennumber,'(i3,".struct")') i
          write(opennumber1,'(i3,".scf")') i
        endif
        fname = fname(1:lnam)//opennumber
        fname1 = fname(1:lnam)//opennumber1
        write(*,'(2a20)')fname,fname1
        open(20,file=fname,status='old',form='formatted',err=1002)
        open(10,file=fname1,status='old',form='formatted',err=1003)
        open(3,file='scf-forces.out')
!
! reading structdata from *.struct
        call readstruct 
        allocate(forces(3,nat))
        allocate(forces_new(3,ndif))
        forces = 0.0d0
        forces_new = 0.0d0
!        write(*,'(a5,i4,a10,i4)')'nat =',nat,'ndif =',ndif
!  
! reading displacements from 'FORCES_old'
        read(1,'(a)')line
        write(2,'(a)')line
        do m = 1,nat
          read(1,*)
        enddo
!
! reading forces from *.scf and write them to scf-forces.out
        do
          read(10,'(A)',end=102,err=1003)line
          length=len(trim(line))
          line1=adjustl(line)
          where1=index(line1,'FGL')
!          where2=index(line1,'total forces')
          where2=index(line1,'total forces')
          if (where1.ne.0.and.where2.ne.0) then
            write(3,'(A)')line(1:length)
          endif
        enddo
102     continue
        rewind(3)
!
        call rotdef(iz,tau,iord,nat,pos,ndif,rotij,tauij,mult,lattic) 
        nindex = 0
        do j = 1,nat
          read(3,'(29X,3F16.9)')forces(1:3,j)
          do k = 1,mult(j)
            nindex=nindex+1
            forces_new(1:3,nindex)=MATMUL(forces(1:3,j), &
                               rotij(1:3,1:3,nindex))
            write(2,'(3f16.10)')(forces_new(m,nindex),m=1,3)
          enddo 
        enddo 
        close(3)
        close(10)
        close(20)
        deallocate(forces,forces_new)    
      enddo
      stop 
!
 1001 write(*,*)'error in opening FORCES or FORCES_new' 
 1002 write(*,*)'error in opening *.struct'
 1003 write(*,*)'error in opening *.scf'
! 
      close(1)
      close(2)
      end

!
!    iz(3,3,iord) - symmetry operations (matrixes in the end of "*.struct")
!    tau(3,iord) - translational vectors (vectors in the end of "*.struct")
!    iord - number of symmetry operations (number in the end of "*.struct")
!    nat - number of inequivalent atoms
!    pos1(3,ndif) - coordinates (x,y,z) of all atoms
!    ndif - total number of atoms
!    rotij(3,3,ndif) - rotational matrix
!    tauij(3,ndif) - translational matrix
!    mult(nat) - multiplicity of inequivalent atoms
!    lattic - type of lattice
!
